@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef __PEAS_EXTENSION_SET_H__
@@ -53,14 +53,31 @@ struct _PeasExtensionSet {
   PeasExtensionSetPrivate *priv;
 };
 
+/**
+ * PeasExtensionSetClass:
+ * @parent_class: The parent class.
+ * @call: The VFunc for peas_extension_set_call().
+ * @extension_added: Signal class handler for the
+ *                   #PeasExtensionSet::extension-added signal.
+ * @extension_removed: Signal class handler for the
+ *                   #PeasExtensionSet::extension-removed signal.
+ *
+ * The class structure for #PeasExtensionSet.
+ */
 struct _PeasExtensionSetClass {
   GObjectClass parent_class;
 
   /* Virtual public methods */
+#ifndef PEAS_DISABLE_DEPRECATED
   gboolean   (*call)                      (PeasExtensionSet *set,
                                            const gchar      *method_name,
                                            GIArgument       *args);
+#else
+  /*< private >*/
+  gpointer __DEPRECATED_call;
+#endif
 
+  /*< public >*/
   /* Signals */
   void       (*extension_added)           (PeasExtensionSet *set,
                                            PeasPluginInfo   *info,
@@ -68,13 +85,34 @@ struct _PeasExtensionSetClass {
   void       (*extension_removed)         (PeasExtensionSet *set,
                                            PeasPluginInfo   *info,
                                            PeasExtension    *exten);
+
+  /*< private >*/
+  gpointer padding[8];
 };
 
+/**
+ * PeasExtensionSetForeachFunc:
+ * @set: A #PeasExtensionSet.
+ * @info: A #PeasPluginInfo.
+ * @exten: A #PeasExtension.
+ * @data: Optional data passed to the function.
+ *
+ * This function is passed to peas_extension_set_foreach() and
+ * will be called for each extension in @set.
+ *
+ * Since: 1.2
+ */
+typedef void (*PeasExtensionSetForeachFunc) (PeasExtensionSet *set,
+                                             PeasPluginInfo   *info,
+                                             PeasExtension    *exten,
+                                             gpointer          data);
+
 /*
  * Public methods
  */
 GType              peas_extension_set_get_type    (void)  G_GNUC_CONST;
 
+#ifndef PEAS_DISABLE_DEPRECATED
 gboolean           peas_extension_set_call        (PeasExtensionSet *set,
                                                    const gchar      *method_name,
                                                    ...);
@@ -84,6 +122,11 @@ gboolean           peas_extension_set_call_valist (PeasExtensionSet *set,
 gboolean           peas_extension_set_callv       (PeasExtensionSet *set,
                                                    const gchar      *method_name,
                                                    GIArgument       *args);
+#endif
+
+void               peas_extension_set_foreach     (PeasExtensionSet *set,
+                                                   PeasExtensionSetForeachFunc func,
+                                                   gpointer          data);
 
 PeasExtension     *peas_extension_set_get_extension (PeasExtensionSet *set,
                                                      PeasPluginInfo   *info);
