@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,10 +24,7 @@
 #endif
 
 #include <seed.h>
-/* FIXME: Seed Bug 624560 */
-SeedValue seed_value_from_gvalue (SeedContext    ctx,
-                                  GValue        *gval,
-                                  SeedException *exception);
+#include <JavaScriptCore/JavaScript.h>
 
 #include "peas-plugin-loader-seed.h"
 #include "peas-extension-seed.h"
@@ -135,8 +132,7 @@ peas_plugin_loader_seed_create_extension (PeasPluginLoader *loader,
 {
   PeasPluginLoaderSeed *sloader = PEAS_PLUGIN_LOADER_SEED (loader);
   SeedInfo *sinfo;
-  SeedValue extension_methods, extension;
-  gchar **property_names;
+  SeedValue extension_ctor, extension;
   guint i, j;
   SeedValue value;
   GValue gvalue = { 0 };
@@ -144,36 +140,55 @@ peas_plugin_loader_seed_create_extension (PeasPluginLoader *loader,
   sinfo = (SeedInfo *) g_hash_table_lookup (sloader->loaded_plugins, info);
 
   /* FIXME: instantiate new object and pass the parameters */
-  extension_methods = seed_object_get_property (sinfo->context,
-                                                sinfo->extensions,
-                                                g_type_name (exten_type));
-  if (!extension_methods ||
-      seed_value_is_undefined (sinfo->context, extension_methods) ||
-      seed_value_is_null (sinfo->context, extension_methods))
+  extension_ctor = seed_object_get_property (sinfo->context,
+                                             sinfo->extensions,
+                                             g_type_name (exten_type));
+  if (!extension_ctor ||
+      seed_value_is_undefined (sinfo->context, extension_ctor) ||
+      seed_value_is_null (sinfo->context, extension_ctor))
     return NULL;
 
-  if (!seed_value_is_object (sinfo->context, extension_methods))
+  if (!seed_value_is_object (sinfo->context, extension_ctor))
     {
       g_warning ("Extension '%s' in plugin '%s' is not a Seed object",
                  g_type_name (exten_type), peas_plugin_info_get_module_name (info));
       return NULL;
     }
 
-  /* Copy the original extension_methods object to a new specific object. */
-  extension = seed_make_object (sinfo->context, NULL, NULL);
-  property_names = seed_object_copy_property_names (sinfo->context,
-                                                    extension_methods);
-  for (i = 0; property_names[i] != NULL; i++)
-    {
-      SeedValue value;
+  /* Instantiate the ctor object into a new specific object. */
+  extension = JSObjectCallAsConstructor (sinfo->context, extension_ctor, 0, NULL, NULL);
 
-      value = seed_object_get_property (sinfo->context,
-                                        extension_methods,
-                                        property_names[i]);
-      seed_object_set_property (sinfo->context,
-                                extension,
-                                property_names[i],
-                                value);
+  if (extension == NULL)
+    {
+#ifndef PEAS_DISABLE_DEPRECATED_FEATURES
+      gchar **property_names;
+
+      g_warning ("DEPRECATION WARNING: Extension '%s' in plugin '%s' is not a valid "
+                 "constructor function. Support for extension initialization by array "
+                 "copy will be dropped soon.",
+                 g_type_name (exten_type), peas_plugin_info_get_module_name (info));
+
+      extension = seed_make_object (sinfo->context, NULL, NULL);
+      property_names = seed_object_copy_property_names (sinfo->context,
+                                                        extension_ctor);
+      for (i = 0; property_names[i] != NULL; i++)
+        {
+          SeedValue value;
+          value = seed_object_get_property (sinfo->context,
+                                            extension_ctor,
+                                            property_names[i]);
+          seed_object_set_property (sinfo->context,
+                                    extension,
+                                    property_names[i],
+                                    value);
+        }
+
+        g_strfreev (property_names);
+#else
+      g_warning ("Extension '%s' in plugin '%s' is not a valid constructor",
+                 g_type_name (exten_type), peas_plugin_info_get_module_name (info));
+      return NULL;
+#endif
     }
 
   /* Set the properties as well, cannot use
@@ -203,8 +218,6 @@ peas_plugin_loader_seed_create_extension (PeasPluginLoader *loader,
       g_free (key);
     }
 
-  g_strfreev (property_names);
-
   /* Set the plugin info as an attribute of the instance */
   g_value_init (&gvalue, PEAS_TYPE_PLUGIN_INFO);
   g_value_set_boxed (&gvalue, info);
